/*******************************************************************
*
*  DESCRIPTION: Atomic Model for Controller1
*
*  AUTHOR: Yuri Boiko
*
*  EMAIL: yuri.boiko@rocketmail.com
*
*  DATE: 27/10/2007
*
*******************************************************************/

#include "controller1.h"
#include "message.h"
#include "mainsimu.h"     // MainSimulator::Instance().getParameter( ... )

Controller1::Controller1( const string &name ) :
	Atomic( name )
	, m1_in ( addInputPort( "m1_in" ) ) // turn on by incoming message 
	, m1_inCount( addInputPort( "m1_inCount" ) )
	, m1_outFire( addOutputPort( "m1_outFire" ))//Neuron_Fires
	, m1_outOff( addOutputPort( "m1_outOff" )) // TIMER_OFF
	, cycleTime( 0, 0, 0, 1 )
	
{
string time( MainSimulator::Instance().getParameter( description(), "cycleTime" ) ) ;
if ( time != "" )
	cycleTime = time ;
}

Controller1::~Controller1()
{
}

Model& Controller1::initFunction()
{
//	this-> passivate();
	holdIn( passive, Time::Zero );
    count = 0;
	return *this;
}

Model& Controller1::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == m1_in)
	{
		if(  this->state() == passive )
		{
			holdIn( Atomic::active, cycleTime );
		}
		else passivate();
	}
	else if( msg.port() == m1_inCount)
	{
		if(  this->state() == active )
		{
		count = static_cast < int > (msg.value());
		holdIn( active, Time::Zero );
		}
		else count = 0;
	}
	return *this;
}

Model& Controller1::internalFunction( const InternalMessage &msg )
{
	passivate();
	return *this;
}

Model& Controller1::outputFunction( const InternalMessage &msg )
{
	if( this->state() == active )
	{
	sendOutput( msg.time(), m1_outOff, 1.0 ); 
	   if (count <= 8 && count >= 5 )
	   {sendOutput( msg.time(), m1_outFire, 1.0 ); 
	   	count = 0;}
	   else 
	   {sendOutput( msg.time(), m1_outFire, 0.0 ); 
	   	count = 0;}
	passivate();
	}

	return *this;
}
