/*******************************************************************
*
*  DESCRIPTION: Atomic Model for Controller2 for time-slot [9,12]
*
*  AUTHOR: Yuri Boiko
*
*  EMAIL: yuri.boiko@rocketmail.com
*
*  DATE: 27/12/2007
*
*******************************************************************/

#include "controller2.h"
#include "message.h"
#include "mainsimu.h"     // MainSimulator::Instance().getParameter( ... )

Controller2::Controller2( const string &name ) :
	Atomic( name )
	, m2_in ( addInputPort( "m2_in" ) ) // turn on by incoming message 
	, m2_inCount( addInputPort( "m2_inCount" ) )
	, m2_outFire( addOutputPort( "m2_outFire" ))//Neuron_Fires
	, m2_outOff( addOutputPort( "m2_outOff" )) // TIMER_OFF
	, cycleTime( 0, 0, 0, 1 )
	
{
string time( MainSimulator::Instance().getParameter( description(), "cycleTime" ) ) ;
if ( time != "" )
	cycleTime = time ;
}

Controller2::~Controller2()
{
}

Model& Controller2::initFunction()
{
//	this-> passivate();
	holdIn( passive, Time::Zero );
    count = 0;
	return *this;
}

Model& Controller2::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == m2_in)
	{
		if(  this->state() == passive )
		{
			holdIn( Atomic::active, cycleTime );
		}
		else passivate();
	}
	else if( msg.port() == m2_inCount)
	{
		if(  this->state() == active )
		{
		count = static_cast < int > (msg.value());
		holdIn( active, Time::Zero );
		}
		else count = 0;
	}
	return *this;
}

Model& Controller2::internalFunction( const InternalMessage &msg )
{
	passivate();
	return *this;
}

Model& Controller2::outputFunction( const InternalMessage &msg )
{
	if( this->state() == active )
	{
	sendOutput( msg.time(), m2_outOff, 1.0 ); 
	   if (count <= 12 && count >= 9 )
	   {sendOutput( msg.time(), m2_outFire, 1.0 ); 
	   	count = 0;}
	   else 
	   {sendOutput( msg.time(), m2_outFire, 0.0 ); 
	   	count = 0;}
	passivate();
	}

	return *this;
}
